@@ -43,6 +43,15 @@ const char version_etc_copyright[] =
 
 #define AUTHORS "Christian Egli"
 
+#define DIRECTION_FORWARD 0
+#define DIRECTION_BACKWARD 1
+#define DIRECTION_BOTH 2
+#define DIRECTION_DEFAULT DIRECTION_FORWARD
+
+#define HYPHENATION_OFF 0
+#define HYPHENATION_ON 1
+#define HYPHENATION_DEFAULT HYPHENATION_OFF
+
 static void
 print_help(void) {
 	printf("\
@@ -85,9 +94,6 @@ const char *encoding_names[] = { "YAML_ANY_ENCODING", "YAML_UTF8_ENCODING",
 
 const char *inline_table_prefix = "checkyaml_inline_";
 
-yaml_parser_t parser;
-yaml_event_t event;
-
 char *file_name;
 
 int errors = 0;
@@ -95,7 +101,7 @@ int count = 0;
 
 static char const **emph_classes = NULL;
 
-void
+static void
 simple_error(const char *msg, yaml_parser_t *parser, yaml_event_t *event) {
 	error_at_line(EXIT_FAILURE, 0, file_name,
 			event->start_mark.line ? event->start_mark.line + 1
@@ -103,22 +109,21 @@ simple_error(const char *msg, yaml_parser_t *parser, yaml_event_t *event) {
 			"%s", msg);
 }
 
-void
+static void
 yaml_parse_error(yaml_parser_t *parser) {
 	error_at_line(EXIT_FAILURE, 0, file_name, parser->problem_mark.line + 1, "%s",
 			parser->problem);
 }
 
-void
+static void
 yaml_error(yaml_event_type_t expected, yaml_event_t *event) {
 	error_at_line(EXIT_FAILURE, 0, file_name, event->start_mark.line + 1,
 			"Expected %s (actual %s)", event_names[expected], event_names[event->type]);
 }
 
-char *
+static char *
 read_table_query(yaml_parser_t *parser, const char **table_file_name_check) {
 	yaml_event_t event;
-	int parse_error = 1;
 	char *query_as_string = malloc(sizeof(char) * MAXSTRING);
 	char *p = query_as_string;
 	query_as_string[0] = '\0';
@@ -157,7 +162,7 @@ read_table_query(yaml_parser_t *parser, const char **table_file_name_check) {
 	return query_as_string;
 }
 
-char *
+static char *
 read_table(yaml_event_t *start_event, yaml_parser_t *parser, const char *display_table) {
 	char *table = NULL;
 	if (start_event->type != YAML_SCALAR_EVENT ||
@@ -261,13 +266,13 @@ read_table(yaml_event_t *start_event, yaml_parser_t *parser, const char *display
 	return table;
 }
 
-void
+static void
 read_flags(yaml_parser_t *parser, int *direction, int *hyphenation) {
 	yaml_event_t event;
 	int parse_error = 1;
 
-	*direction = 0;
-	*hyphenation = 0;
+	*direction = DIRECTION_DEFAULT;
+	*hyphenation = HYPHENATION_DEFAULT;
 
 	if (!yaml_parser_parse(parser, &event) || (event.type != YAML_MAPPING_START_EVENT))
 		yaml_error(YAML_MAPPING_START_EVENT, &event);
@@ -281,11 +286,13 @@ read_flags(yaml_parser_t *parser, int *direction, int *hyphenation) {
 			if (!yaml_parser_parse(parser, &event) || (event.type != YAML_SCALAR_EVENT))
 				yaml_error(YAML_SCALAR_EVENT, &event);
 			if (!strcmp((const char *)event.data.scalar.value, "forward")) {
-				*direction = 0;
+				*direction = DIRECTION_FORWARD;
 			} else if (!strcmp((const char *)event.data.scalar.value, "backward")) {
-				*direction = 1;
+				*direction = DIRECTION_BACKWARD;
+			} else if (!strcmp((const char *)event.data.scalar.value, "bothDirections")) {
+				*direction = DIRECTION_BOTH;
 			} else if (!strcmp((const char *)event.data.scalar.value, "hyphenate")) {
-				*hyphenation = 1;
+				*hyphenation = HYPHENATION_ON;
 			} else {
 				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
 						"Testmode '%s' not supported\n", event.data.scalar.value);
@@ -300,7 +307,7 @@ read_flags(yaml_parser_t *parser, int *direction, int *hyphenation) {
 	yaml_event_delete(&event);
 }
 
-int
+static int
 read_xfail(yaml_parser_t *parser) {
 	yaml_event_t event;
 	/* assume xfail true if there is an xfail key */
@@ -314,7 +321,7 @@ read_xfail(yaml_parser_t *parser) {
 	return xfail;
 }
 
-translationModes
+static translationModes
 read_mode(yaml_parser_t *parser) {
 	yaml_event_t event;
 	translationModes mode = 0;
@@ -353,8 +360,8 @@ read_mode(yaml_parser_t *parser) {
 	return mode;
 }
 
-int
-parse_number(const char *number, char *name, int file_line) {
+static int
+parse_number(const char *number, const char *name, int file_line) {
 	char *tail;
 	errno = 0;
 
@@ -368,7 +375,7 @@ parse_number(const char *number, char *name, int file_line) {
 	return val;
 }
 
-int *
+static int *
 read_inPos(yaml_parser_t *parser, int translen) {
 	int *pos = malloc(sizeof(int) * translen);
 	int i = 0;
@@ -400,7 +407,7 @@ read_inPos(yaml_parser_t *parser, int translen) {
 	return pos;
 }
 
-int *
+static int *
 read_outPos(yaml_parser_t *parser, int wrdlen, int translen) {
 	int *pos = malloc(sizeof(int) * wrdlen);
 	int i = 0;
@@ -432,7 +439,7 @@ read_outPos(yaml_parser_t *parser, int wrdlen, int translen) {
 	return pos;
 }
 
-void
+static void
 read_cursorPos(yaml_parser_t *parser, int *cursorPos, int *expected_cursorPos, int wrdlen,
 		int translen) {
 	yaml_event_t event;
@@ -484,7 +491,7 @@ read_cursorPos(yaml_parser_t *parser, int *cursorPos, int *expected_cursorPos, i
 	}
 }
 
-void
+static void
 read_typeform_string(yaml_parser_t *parser, formtype *typeform, typeforms kind, int len) {
 	yaml_event_t event;
 	int typeform_len;
@@ -500,7 +507,7 @@ read_typeform_string(yaml_parser_t *parser, formtype *typeform, typeforms kind,
 	yaml_event_delete(&event);
 }
 
-formtype *
+static formtype *
 read_typeforms(yaml_parser_t *parser, int len) {
 	yaml_event_t event;
 	formtype *typeform = calloc(len, sizeof(formtype));
@@ -546,10 +553,10 @@ read_typeforms(yaml_parser_t *parser, int len) {
 	return typeform;
 }
 
-void
-read_options(yaml_parser_t *parser, int wordLen, int translationLen, int *xfail,
-		translationModes *mode, formtype **typeform, int **inPos, int **outPos,
-		int *cursorPos, int *cursorOutPos) {
+static void
+read_options(yaml_parser_t *parser, int direction, int wordLen, int translationLen,
+		int *xfail, translationModes *mode, formtype **typeform, int **inPos,
+		int **outPos, int *cursorPos, int *cursorOutPos, int *maxOutputLen) {
 	yaml_event_t event;
 	char *option_name;
 	int parse_error = 1;
@@ -572,6 +579,10 @@ read_options(yaml_parser_t *parser, int wordLen, int translationLen, int *xfail,
 			yaml_event_delete(&event);
 			*mode = read_mode(parser);
 		} else if (!strcmp(option_name, "typeform")) {
+			if (direction != 0) {
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"typeforms only supported with testmode 'forward'\n");
+			}
 			yaml_event_delete(&event);
 			*typeform = read_typeforms(parser, wordLen);
 		} else if (!strcmp(option_name, "inputPos")) {
@@ -581,8 +592,32 @@ read_options(yaml_parser_t *parser, int wordLen, int translationLen, int *xfail,
 			yaml_event_delete(&event);
 			*outPos = read_outPos(parser, wordLen, translationLen);
 		} else if (!strcmp(option_name, "cursorPos")) {
+			if (direction == 2) {
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"cursorPos not supported with testmode 'bothDirections'\n");
+			}
 			yaml_event_delete(&event);
 			read_cursorPos(parser, cursorPos, cursorOutPos, wordLen, translationLen);
+		} else if (!strcmp(option_name, "maxOutputLength")) {
+			if (direction == 2) {
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"maxOutputLength not supported with testmode 'bothDirections'\n");
+			}
+			yaml_event_delete(&event);
+			if (!yaml_parser_parse(parser, &event) || (event.type != YAML_SCALAR_EVENT))
+				yaml_error(YAML_SCALAR_EVENT, &event);
+			*maxOutputLen = parse_number((const char *)event.data.scalar.value,
+					"Maximum output length", event.start_mark.line + 1);
+			if (*maxOutputLen <= 0)
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"Maximum output length (%i) must be a positive number\n",
+						*maxOutputLen);
+			if (*maxOutputLen < translationLen)
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"Expected translation length (%i) must not exceed maximum output "
+						"length (%i)\n",
+						translationLen, *maxOutputLen);
+			yaml_event_delete(&event);
 		} else {
 			error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
 					"Unsupported option %s", option_name);
@@ -595,7 +630,7 @@ read_options(yaml_parser_t *parser, int wordLen, int translationLen, int *xfail,
 }
 
 /* see http://stackoverflow.com/questions/5117393/utf-8-strings-length-in-linux-c */
-int
+static int
 my_strlen_utf8_c(char *s) {
 	int i = 0, j = 0;
 	while (s[i]) {
@@ -605,7 +640,7 @@ my_strlen_utf8_c(char *s) {
 	return j;
 }
 
-void
+static void
 read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation) {
 	yaml_event_t event;
 	char *description = NULL;
@@ -618,6 +653,7 @@ read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation)
 	int *outPos = NULL;
 	int cursorPos = -1;
 	int cursorOutPos = -1;
+	int maxOutputLen = -1;
 
 	if (!yaml_parser_parse(parser, &event) || (event.type != YAML_SCALAR_EVENT))
 		simple_error("Word expected", parser, &event);
@@ -647,8 +683,9 @@ read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation)
 
 	if (event.type == YAML_MAPPING_START_EVENT) {
 		yaml_event_delete(&event);
-		read_options(parser, my_strlen_utf8_c(word), my_strlen_utf8_c(translation),
-				&xfail, &mode, &typeform, &inPos, &outPos, &cursorPos, &cursorOutPos);
+		read_options(parser, direction, my_strlen_utf8_c(word),
+				my_strlen_utf8_c(translation), &xfail, &mode, &typeform, &inPos, &outPos,
+				&cursorPos, &cursorOutPos, &maxOutputLen);
 
 		if (!yaml_parser_parse(parser, &event) || (event.type != YAML_SEQUENCE_END_EVENT))
 			yaml_error(YAML_SEQUENCE_END_EVENT, &event);
@@ -661,7 +698,7 @@ read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation)
 	int result = 0;
 	char **table = tables;
 	while (*table) {
-		if (hyphenation) {
+		if (hyphenation == HYPHENATION_ON) {
 			result |= check_hyphenation(*table, word, translation);
 		} else {
 			// FIXME: Note that the typeform array was constructed using the
@@ -672,7 +709,8 @@ read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation)
 			result |= check(*table, word, translation, .typeform = typeform, .mode = mode,
 					.expected_inputPos = inPos, .expected_outputPos = outPos,
 					.cursorPos = cursorPos, .expected_cursorPos = cursorOutPos,
-					.direction = direction, .diagnostics = !xfail);
+					.max_outlen = maxOutputLen, .direction = direction,
+					.diagnostics = !xfail);
 		}
 		table++;
 	}
@@ -693,7 +731,7 @@ read_test(yaml_parser_t *parser, char **tables, int direction, int hyphenation)
 	free(outPos);
 }
 
-void
+static void
 read_tests(yaml_parser_t *parser, char **tables, int direction, int hyphenation) {
 	yaml_event_t event;
 	if (!yaml_parser_parse(parser, &event) || (event.type != YAML_SEQUENCE_START_EVENT))
@@ -876,34 +914,43 @@ main(int argc, char *argv[]) {
 
 		if (event.type != YAML_SCALAR_EVENT) yaml_error(YAML_SCALAR_EVENT, &event);
 
-		int direction = 0;
-		int hyphenation = 0;
-		if (!strcmp((const char *)event.data.scalar.value, "flags")) {
-			yaml_event_delete(&event);
-			read_flags(&parser, &direction, &hyphenation);
+		int haveRunTests = 0;
+		while (1) {
+			int direction = DIRECTION_DEFAULT;
+			int hyphenation = HYPHENATION_DEFAULT;
+			if (!strcmp((const char *)event.data.scalar.value, "flags")) {
+				yaml_event_delete(&event);
+				read_flags(&parser, &direction, &hyphenation);
 
-			if (!yaml_parser_parse(&parser, &event) ||
-					(event.type != YAML_SCALAR_EVENT) ||
-					strcmp((const char *)event.data.scalar.value, "tests")) {
-				simple_error("tests expected", &parser, &event);
+				if (!yaml_parser_parse(&parser, &event) ||
+						(event.type != YAML_SCALAR_EVENT) ||
+						strcmp((const char *)event.data.scalar.value, "tests")) {
+					simple_error("tests expected", &parser, &event);
+				}
+				yaml_event_delete(&event);
+				read_tests(&parser, tables, direction, hyphenation);
+				haveRunTests = 1;
+
+			} else if (!strcmp((const char *)event.data.scalar.value, "tests")) {
+				yaml_event_delete(&event);
+				read_tests(&parser, tables, direction, hyphenation);
+				haveRunTests = 1;
+			} else {
+				if (haveRunTests) {
+					break;
+				} else {
+					simple_error("flags or tests expected", &parser, &event);
+				}
 			}
-			yaml_event_delete(&event);
-			read_tests(&parser, tables, direction, hyphenation);
-
-		} else if (!strcmp((const char *)event.data.scalar.value, "tests")) {
-			yaml_event_delete(&event);
-			read_tests(&parser, tables, direction, hyphenation);
-		} else {
-			simple_error("flags or tests expected", &parser, &event);
+			if (!yaml_parser_parse(&parser, &event))
+				error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
+						"Expected table, flags, tests or %s (actual %s)",
+						event_names[YAML_MAPPING_END_EVENT], event_names[event.type]);
+			if (event.type != YAML_SCALAR_EVENT) break;
 		}
 
 		char **p = tables;
 		while (*p) free(*(p++));
-
-		if (!yaml_parser_parse(&parser, &event))
-			error_at_line(EXIT_FAILURE, 0, file_name, event.start_mark.line + 1,
-					"Expected table or %s (actual %s)",
-					event_names[YAML_MAPPING_END_EVENT], event_names[event.type]);
 	}
 	if (event.type != YAML_MAPPING_END_EVENT) yaml_error(YAML_MAPPING_END_EVENT, &event);
 	yaml_event_delete(&event);
