@@ -33,8 +33,6 @@
 #include "unistr.h"
 #include "version-etc.h"
 
-#define BUFSIZE MAXSTRING - 4
-
 static int forward_flag = 0;
 static int backward_flag = 0;
 
@@ -53,20 +51,20 @@ const char version_etc_copyright[] =
 
 static void
 translate_input(int forward_translation, char *table_name) {
-	char charbuf[BUFSIZE];
+	char charbuf[MAXSTRING];
 	uint8_t *outputbuf;
 	size_t outlen;
-	widechar inbuf[BUFSIZE];
-	widechar transbuf[BUFSIZE];
+	widechar inbuf[MAXSTRING];
+	widechar transbuf[MAXSTRING];
 	int inlen;
 	int translen;
 	int k;
 	int ch = 0;
 	int result;
 	while (1) {
-		translen = BUFSIZE;
+		translen = MAXSTRING;
 		k = 0;
-		while ((ch = fgetc(input)) != '\n' && ch != EOF && k < BUFSIZE - 1)
+		while ((ch = fgetc(input)) != '\n' && ch != EOF && k < MAXSTRING - 1)
 			charbuf[k++] = ch;
 		if (ch == EOF && k == 0) break;
 		charbuf[k] = 0;
